@@ -97,8 +97,22 @@ the second with the current group name.")
 (defvar gnus-bug-create-help-buffer t
   "*Should we create the *Gnus Help Bug* buffer?")
 
+(defvar gnus-posting-styles nil
+  "*Alist of styles to use when posting.")
+
+(defvar gnus-posting-style-alist
+  '((organization . message-user-organization)
+    (signature . message-signature)
+    (signature-file . message-signature-file)
+    (address . user-mail-address)
+    (name . user-full-name))
+  "*Mapping from style parameters to variables.")
+
 ;;; Internal variables.
 
+(defvar gnus-inhibit-posting-styles nil
+  "Inhibit the use of posting styles.")
+
 (defvar gnus-message-buffer "*Mail Gnus*")
 (defvar gnus-article-copy nil)
 (defvar gnus-last-posting-server nil)
@@ -174,9 +188,11 @@ Thank you for your help in stamping out bugs.
 	   (,article (and gnus-article-reply (gnus-summary-article-number)))
 	   (,group gnus-newsgroup-name)
 	   (message-header-setup-hook
-	    (copy-sequence message-header-setup-hook)))
+	    (copy-sequence message-header-setup-hook))
+	   (message-mode-hook (copy-sequence message-mode-hook)))
        (add-hook 'message-header-setup-hook 'gnus-inews-insert-gcc)
        (add-hook 'message-header-setup-hook 'gnus-inews-insert-archive-gcc)
+       (add-hook 'message-mode-hook 'gnus-configure-posting-styles)
        (unwind-protect
 	   (progn
 	     ,@forms)
@@ -186,6 +202,7 @@ Thank you for your help in stamping out bugs.
 	      (cons ,group ,article))
 	 (make-local-variable 'gnus-newsgroup-name)
 	 (gnus-run-hooks 'gnus-message-setup-hook))
+       (gnus-add-buffer)
        (gnus-configure-windows ,config t)
        (set-buffer-modified-p nil))))
 
@@ -335,10 +352,8 @@ header line with the old Message-ID."
   ;; this copy is in the buffer gnus-article-copy.
   ;; if ARTICLE-BUFFER is nil, gnus-article-buffer is used
   ;; this buffer should be passed to all mail/news reply/post routines.
-  (setq gnus-article-copy (get-buffer-create " *gnus article copy*"))
+  (setq gnus-article-copy (gnus-get-buffer-create " *gnus article copy*"))
   (buffer-disable-undo gnus-article-copy)
-  (or (memq gnus-article-copy gnus-buffer-list)
-      (push gnus-article-copy gnus-buffer-list))
   (let ((article-buffer (or article-buffer gnus-article-buffer))
 	end beg contents)
     (if (not (and (get-buffer article-buffer)
@@ -411,6 +426,7 @@ header line with the old Message-ID."
 	    (if post
 		(message-news (or to-group group))
 	      (set-buffer gnus-article-copy)
+	      (gnus-msg-treat-broken-reply-to)
 	      (message-followup (if (or newsgroup-p force-news) nil to-group)))
 	  ;; The is mail.
 	  (if post
@@ -496,9 +512,11 @@ If SILENT, don't prompt the user."
 	  method-alist))))
      ;; Override normal method.
      ((and (eq gnus-post-method 'current)
+	   (not (eq (car group-method) 'nndraft))
 	   (not arg))
       group-method) 
-     (gnus-post-method
+     ((and gnus-post-method
+	   (not (eq gnus-post-method 'current)))
       gnus-post-method)
      ;; Use the normal select method.
      (t gnus-select-method))))
@@ -585,7 +603,8 @@ automatically."
   ;; Stripping headers should be specified with mail-yank-ignored-headers.
   (when yank
     (gnus-summary-goto-subject (car yank)))
-  (let ((gnus-article-reply t))
+  (let ((gnus-article-reply t)
+	(gnus-inhibit-posting-styles t))
     (gnus-setup-message (if yank 'reply-yank 'reply)
       (gnus-summary-select-article)
       (set-buffer (gnus-copy-article-buffer))
@@ -836,7 +855,7 @@ The source file has to be in the Emacs load path."
     (sit-for 0)
     ;; Go through all the files looking for non-default values for variables.
     (save-excursion
-      (set-buffer (get-buffer-create " *gnus bug info*"))
+      (set-buffer (gnus-get-buffer-create " *gnus bug info*"))
       (buffer-disable-undo (current-buffer))
       (while files
 	(erase-buffer)
@@ -1046,6 +1065,89 @@ this is a reply."
 		(insert " ")))
 	    (insert "\n")))))))
 
+;;; Posting styles.
+
+(defvar gnus-message-style-insertions nil)
+
+(defun gnus-configure-posting-styles ()
+  "Configure posting styles according to `gnus-posting-styles'."
+  (unless gnus-inhibit-posting-styles
+    (let ((styles gnus-posting-styles)
+	  (gnus-newsgroup-name (or gnus-newsgroup-name ""))
+	  style match variable attribute value value-value)
+      (make-local-variable 'gnus-message-style-insertions)
+      ;; Go through all styles and look for matches.
+      (while styles
+	(setq style (pop styles)
+	      match (pop style))
+	(when (cond ((stringp match)
+		     ;; Regexp string match on the group name.
+		     (string-match match gnus-newsgroup-name))
+		    ((or (symbolp match)
+			 (gnus-functionp match))
+		     (cond ((gnus-functionp match)
+			    ;; Function to be called.
+			    (funcall match))
+			   ((boundp match)
+			    ;; Variable to be checked.
+			    (symbol-value match))))
+		    ((listp match)
+		     ;; This is a form to be evaled.
+		     (eval match)))
+	  ;; We have a match, so we set the variables.
+	  (while style
+	    (setq attribute (pop style)
+		  value (cadr attribute)
+		  variable nil)
+	    ;; We find the variable that is to be modified.
+	    (if (and (not (stringp (car attribute)))
+		     (not (eq 'body (car attribute)))
+		     (not (setq variable
+				(cdr (assq (car attribute) 
+					   gnus-posting-style-alist)))))
+		(message "Couldn't find attribute %s" (car attribute))
+	      ;; We get the value.
+	      (setq value-value
+		    (cond ((stringp value)
+			   value)
+			  ((or (symbolp value)
+			       (gnus-functionp value))
+			   (cond ((gnus-functionp value)
+				  (funcall value))
+				 ((boundp value)
+				  (symbol-value value))))
+			  ((listp value)
+			   (eval value))))
+	      (if variable
+		  ;; This is an ordinary variable.
+		  (set (make-local-variable variable) value-value)
+		;; This is either a body or a header to be inserted in the
+		;; message.
+		(when value-value
+		  (let ((attr (car attribute)))
+		    (make-local-variable 'message-setup-hook)
+		    (if (eq 'body attr)
+			(add-hook 'message-setup-hook
+				  `(lambda ()
+				     (save-excursion
+				       (message-goto-body)
+				       (insert ,value-value))))
+		      (add-hook 'message-setup-hook
+				'gnus-message-insert-stylings)
+		      (push (cons (if (stringp attr) attr
+				    (symbol-name attr))
+				  value-value)
+			    gnus-message-style-insertions))))))))))))
+
+(defun gnus-message-insert-stylings ()
+  (let (val)
+    (save-excursion
+      (message-goto-eoh)
+      (while (setq val (pop gnus-message-style-insertions))
+	(when (cdr val)
+	  (insert (car val) ": " (cdr val) "\n"))
+	(gnus-pull (car val) gnus-message-style-insertions)))))
+
 ;;; Allow redefinition of functions.
 
 (gnus-ems-redefine)
